@@ -9,22 +9,33 @@ use std::io;
 
 use md5::{Digest, Md5};
 
-use noodles_bam as bam;
+use noodles_core::Position;
 use noodles_cram as cram;
 use noodles_fasta as fasta;
 use noodles_sam::{
     self as sam,
     header::{self, reference_sequence::Md5Checksum, Program, ReferenceSequence},
 };
+use sam::AlignmentWriter;
 
-static FASTA_DATA: &[u8] = b"\
->sq0
-TTCACCCA
->sq1
-GATCTTACTTTTT
->sq2
-GGCGCCCCGCTGTGCAAAAAT
-";
+fn build_reference_sequences() -> Vec<fasta::Record> {
+    use fasta::record::{Definition, Sequence};
+
+    vec![
+        fasta::Record::new(
+            Definition::new("sq0", None),
+            Sequence::from(b"TTCACCCA".to_vec()),
+        ),
+        fasta::Record::new(
+            Definition::new("sq1", None),
+            Sequence::from(b"GATCTTACTTTTT".to_vec()),
+        ),
+        fasta::Record::new(
+            Definition::new("sq2", None),
+            Sequence::from(b"GGCGCCCCGCTGTGCAAAAAT".to_vec()),
+        ),
+    ]
+}
 
 fn build_header(
     reference_sequence_records: &[fasta::Record],
@@ -37,13 +48,17 @@ fn build_header(
     for record in reference_sequence_records {
         let sequence = record.sequence();
 
+        let name = record.name().parse()?;
+        let len = i32::try_from(sequence.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
         let mut hasher = Md5::new();
         hasher.update(&sequence);
         let md5_checksum = Md5Checksum::from(<[u8; 16]>::from(hasher.finalize()));
 
         let reference_sequence = ReferenceSequence::builder()
-            .set_name(record.name().parse()?)
-            .set_length(sequence.len() as i32)
+            .set_name(name)
+            .set_length(len)
             .set_md5_checksum(md5_checksum)
             .build()?;
 
@@ -54,33 +69,36 @@ fn build_header(
 }
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
-    let reference_sequence_records: Vec<_> = fasta::Reader::new(FASTA_DATA)
-        .records()
-        .collect::<Result<_, _>>()?;
-
-    let header = build_header(&reference_sequence_records)?;
+    let reference_sequences = build_reference_sequences();
 
     let stdout = io::stdout();
     let handle = stdout.lock();
 
-    let mut writer = cram::Writer::new(handle, reference_sequence_records);
+    let header = build_header(&reference_sequences)?;
+
+    let repository = fasta::Repository::new(reference_sequences);
+    let mut writer = cram::Writer::builder(handle)
+        .set_reference_sequence_repository(repository)
+        .build();
+
     writer.write_file_definition()?;
     writer.write_file_header(&header)?;
 
-    let record = cram::Record::builder()
-        .set_bam_flags(sam::record::Flags::empty())
-        .set_flags(cram::record::Flags::QUALITY_SCORES_STORED_AS_ARRAY)
-        .set_reference_sequence_id(bam::record::ReferenceSequenceId::try_from(0)?)
-        .set_alignment_start(sam::record::Position::try_from(1)?)
-        .set_read_length(4)
-        .set_bases(b"TTCA".to_vec())
-        .set_quality_scores(vec![45, 35, 43, 50])
+    let record = sam::Record::builder()
+        .set_flags(sam::record::Flags::empty())
+        .set_reference_sequence_name("sq0".parse()?)
+        .set_position(Position::MIN)
+        .set_cigar("4M".parse()?)
+        .set_sequence("TTCA".parse()?)
+        .set_quality_scores("NDLS".parse()?)
         .build();
 
-    writer.write_record(record)?;
+    writer.write_alignment_record(&header, &record)?;
 
     let record = cram::Record::default();
-    writer.write_record(record)?;
+    writer.write_record(&header, record)?;
+
+    writer.try_finish(&header)?;
 
     Ok(())
 }
