@@ -1,9 +1,8 @@
-use noodles_sam as sam;
+use noodles_core::Position;
 use tokio::io::{self, AsyncRead, AsyncReadExt};
 
 use crate::{
     container::{self, ReferenceSequenceId},
-    num::Itf8,
     r#async::reader::num::{read_itf8, read_ltf8},
 };
 
@@ -11,23 +10,23 @@ pub async fn read_header<R>(reader: &mut R) -> io::Result<container::Header>
 where
     R: AsyncRead + Unpin,
 {
-    let length = reader.read_i32_le().await?;
+    let length = reader.read_i32_le().await.and_then(|n| {
+        usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+    })?;
 
     let reference_sequence_id = read_itf8(reader).await.and_then(|n| {
         ReferenceSequenceId::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
     })?;
 
-    let starting_position_on_the_reference = read_itf8(reader).await.and_then(|n| {
-        if n == 0 {
-            Ok(None)
-        } else {
-            sam::record::Position::try_from(n)
-                .map(Some)
-                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-        }
+    let starting_position_on_the_reference = read_itf8(reader)
+        .await
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+        .map(Position::new)?;
+
+    let alignment_span = read_itf8(reader).await.and_then(|n| {
+        usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
     })?;
 
-    let alignment_span = read_itf8(reader).await?;
     let number_of_records = read_itf8(reader).await?;
     let record_counter = read_ltf8(reader).await?;
     let bases = read_ltf8(reader).await?;
@@ -57,17 +56,21 @@ where
     Ok(builder.build())
 }
 
-async fn read_landmarks<R>(reader: &mut R) -> io::Result<Vec<Itf8>>
+async fn read_landmarks<R>(reader: &mut R) -> io::Result<Vec<usize>>
 where
     R: AsyncRead + Unpin,
 {
     let len = read_itf8(reader).await.and_then(|n| {
         usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
     })?;
+
     let mut landmarks = Vec::with_capacity(len);
 
     for _ in 0..len {
-        let pos = read_itf8(reader).await?;
+        let pos = read_itf8(reader).await.and_then(|n| {
+            usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+        })?;
+
         landmarks.push(pos);
     }
 
@@ -101,7 +104,7 @@ mod tests {
         let expected = container::Header::builder()
             .set_length(144)
             .set_reference_sequence_id(ReferenceSequenceId::try_from(2)?)
-            .set_start_position(sam::record::Position::try_from(3)?)
+            .set_start_position(Position::try_from(3)?)
             .set_alignment_span(5)
             .set_record_count(8)
             .set_record_counter(13)
