@@ -7,11 +7,11 @@ pub use self::{
 };
 
 use std::{
-    borrow::Cow,
     io::{self, Read},
     mem,
 };
 
+use bytes::Bytes;
 use bzip2::read::BzDecoder;
 use flate2::read::GzDecoder;
 use xz2::read::XzDecoder;
@@ -21,7 +21,7 @@ use crate::{
         aac::arith_decode, fqzcomp::fqz_decode, name_tokenizer::decode_names, rans::rans_decode,
         rans_nx16::rans_decode_nx16,
     },
-    num::{itf8, Itf8},
+    num::itf8,
 };
 
 // § 9 End of file container (2020-06-22)
@@ -32,9 +32,9 @@ const EOF_CRC32: u32 = 0x4b_01_63_ee;
 pub struct Block {
     compression_method: CompressionMethod,
     content_type: ContentType,
-    content_id: Itf8,
+    content_id: i32,
     uncompressed_len: usize,
-    data: Vec<u8>,
+    data: Bytes,
     crc32: u32,
 }
 
@@ -49,7 +49,7 @@ impl Block {
         Self::builder()
             .set_content_type(ContentType::CompressionHeader)
             .set_uncompressed_len(EOF_DATA.len())
-            .set_data(EOF_DATA.to_vec())
+            .set_data(Bytes::from_static(&EOF_DATA))
             .set_crc32(EOF_CRC32)
             .build()
     }
@@ -62,7 +62,7 @@ impl Block {
         self.content_type
     }
 
-    pub fn content_id(&self) -> Itf8 {
+    pub fn content_id(&self) -> i32 {
         self.content_id
     }
 
@@ -74,48 +74,48 @@ impl Block {
         &self.data
     }
 
-    pub fn decompressed_data(&self) -> io::Result<Cow<'_, [u8]>> {
+    pub fn decompressed_data(&self) -> io::Result<Bytes> {
         match self.compression_method {
-            CompressionMethod::None => Ok(Cow::from(self.data())),
+            CompressionMethod::None => Ok(self.data.clone()),
             CompressionMethod::Gzip => {
                 let mut reader = GzDecoder::new(self.data());
                 let mut buf = Vec::with_capacity(self.uncompressed_len);
                 reader.read_to_end(&mut buf)?;
-                Ok(Cow::from(buf))
+                Ok(Bytes::from(buf))
             }
             CompressionMethod::Bzip2 => {
                 let mut reader = BzDecoder::new(self.data());
                 let mut buf = Vec::with_capacity(self.uncompressed_len);
                 reader.read_to_end(&mut buf)?;
-                Ok(Cow::from(buf))
+                Ok(Bytes::from(buf))
             }
             CompressionMethod::Lzma => {
                 let mut reader = XzDecoder::new(self.data());
                 let mut buf = Vec::with_capacity(self.uncompressed_len);
                 reader.read_to_end(&mut buf)?;
-                Ok(Cow::from(buf))
+                Ok(Bytes::from(buf))
             }
             CompressionMethod::Rans4x8 => {
                 let mut buf = self.data();
-                rans_decode(&mut buf).map(Cow::from)
+                rans_decode(&mut buf).map(Bytes::from)
             }
             CompressionMethod::RansNx16 => {
                 let mut reader = self.data();
-                rans_decode_nx16(&mut reader, self.uncompressed_len()).map(Cow::from)
+                rans_decode_nx16(&mut reader, self.uncompressed_len()).map(Bytes::from)
             }
             CompressionMethod::AdaptiveArithmeticCoding => {
                 let mut reader = self.data();
-                arith_decode(&mut reader, self.uncompressed_len()).map(Cow::from)
+                arith_decode(&mut reader, self.uncompressed_len()).map(Bytes::from)
             }
             CompressionMethod::Fqzcomp => {
                 let mut reader = self.data();
-                fqz_decode(&mut reader).map(Cow::from)
+                fqz_decode(&mut reader).map(Bytes::from)
             }
             CompressionMethod::NameTokenizer => {
                 let mut reader = self.data();
                 let names = decode_names(&mut reader)?;
                 let data: Vec<_> = names.into_iter().flat_map(|s| s.into_bytes()).collect();
-                Ok(Cow::from(data))
+                Ok(Bytes::from(data))
             }
         }
     }
@@ -126,8 +126,8 @@ impl Block {
             // block content type ID
             + mem::size_of::<u8>()
             + itf8::size_of(self.content_id())
-            + itf8::size_of(self.data.len() as Itf8)
-            + itf8::size_of(self.uncompressed_len() as Itf8)
+            + itf8::size_of(self.data.len() as i32)
+            + itf8::size_of(self.uncompressed_len() as i32)
             + self.data.len()
             // crc32
             + mem::size_of::<u32>()
@@ -140,7 +140,7 @@ mod tests {
 
     #[test]
     fn test_len() {
-        let data = b"noodles".to_vec();
+        let data = Bytes::from_static(b"noodles");
 
         let block = Block::builder()
             .set_content_type(ContentType::ExternalData)
