@@ -1,9 +1,8 @@
 use std::io::{self, Write};
 
-use crate::num::Itf8;
-
 use super::{Block, CompressionMethod, ContentType};
 
+use bytes::Bytes;
 use bzip2::write::BzEncoder;
 use flate2::write::GzEncoder;
 use xz2::write::XzEncoder;
@@ -14,9 +13,9 @@ const DEFAULT_LZMA_COMPRESSION_LEVEL: u32 = 6;
 pub struct Builder {
     compression_method: CompressionMethod,
     content_type: Option<ContentType>,
-    content_id: Itf8,
+    content_id: i32,
     uncompressed_len: usize,
-    data: Vec<u8>,
+    data: Bytes,
     crc32: u32,
 }
 
@@ -31,7 +30,7 @@ impl Builder {
         self
     }
 
-    pub fn set_content_id(mut self, content_id: Itf8) -> Self {
+    pub fn set_content_id(mut self, content_id: i32) -> Self {
         self.content_id = content_id;
         self
     }
@@ -41,7 +40,7 @@ impl Builder {
         self
     }
 
-    pub fn set_data(mut self, data: Vec<u8>) -> Self {
+    pub fn set_data(mut self, data: Bytes) -> Self {
         self.data = data;
         self
     }
@@ -63,7 +62,7 @@ impl Builder {
         self.compression_method = compression_method;
         self.uncompressed_len = data.len();
 
-        self.data = match compression_method {
+        let data = match compression_method {
             CompressionMethod::None => data,
             CompressionMethod::Gzip => {
                 let mut encoder = GzEncoder::new(Vec::new(), flate2::Compression::default());
@@ -86,6 +85,8 @@ impl Builder {
             ),
         };
 
+        self.data = Bytes::from(data);
+
         Ok(self)
     }
 
