@@ -16,8 +16,16 @@ pub enum CompressionMethod {
     Bzip2,
     /// Lempel-Ziv-Markov chain algorithm (LZMA).
     Lzma,
-    /// Ranged asymmetric numeral systems (rANS).
-    Rans,
+    /// Ranged asymmetric numeral systems (4 states, 8-bit renormalization; rANS 4x8).
+    Rans4x8,
+    /// Ranged asymmetric numeral systems (N states, 16-bit renormalization; rANS Nx16).
+    RansNx16,
+    /// Adaptive arithmetic coding.
+    AdaptiveArithmeticCoding,
+    /// fqzcomp.
+    Fqzcomp,
+    /// Name tokenization codec.
+    NameTokenizer,
 }
 
 impl Default for CompressionMethod {
@@ -35,7 +43,7 @@ impl fmt::Display for TryFromByteError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
-            "invalid compression method: expected 0..=4, got {}",
+            "invalid compression method: expected 0..=8, got {}",
             self.0
         )
     }
@@ -50,7 +58,11 @@ impl TryFrom<u8> for CompressionMethod {
             1 => Ok(Self::Gzip),
             2 => Ok(Self::Bzip2),
             3 => Ok(Self::Lzma),
-            4 => Ok(Self::Rans),
+            4 => Ok(Self::Rans4x8),
+            5 => Ok(Self::RansNx16),
+            6 => Ok(Self::AdaptiveArithmeticCoding),
+            7 => Ok(Self::Fqzcomp),
+            8 => Ok(Self::NameTokenizer),
             _ => Err(TryFromByteError(b)),
         }
     }
@@ -63,7 +75,11 @@ impl From<CompressionMethod> for u8 {
             CompressionMethod::Gzip => 1,
             CompressionMethod::Bzip2 => 2,
             CompressionMethod::Lzma => 3,
-            CompressionMethod::Rans => 4,
+            CompressionMethod::Rans4x8 => 4,
+            CompressionMethod::RansNx16 => 5,
+            CompressionMethod::AdaptiveArithmeticCoding => 6,
+            CompressionMethod::Fqzcomp => 7,
+            CompressionMethod::NameTokenizer => 8,
         }
     }
 }
@@ -83,8 +99,27 @@ mod tests {
         assert_eq!(CompressionMethod::try_from(1), Ok(CompressionMethod::Gzip));
         assert_eq!(CompressionMethod::try_from(2), Ok(CompressionMethod::Bzip2));
         assert_eq!(CompressionMethod::try_from(3), Ok(CompressionMethod::Lzma));
-        assert_eq!(CompressionMethod::try_from(4), Ok(CompressionMethod::Rans));
-        assert_eq!(CompressionMethod::try_from(5), Err(TryFromByteError(5)));
+        assert_eq!(
+            CompressionMethod::try_from(4),
+            Ok(CompressionMethod::Rans4x8)
+        );
+        assert_eq!(
+            CompressionMethod::try_from(5),
+            Ok(CompressionMethod::RansNx16)
+        );
+        assert_eq!(
+            CompressionMethod::try_from(6),
+            Ok(CompressionMethod::AdaptiveArithmeticCoding)
+        );
+        assert_eq!(
+            CompressionMethod::try_from(7),
+            Ok(CompressionMethod::Fqzcomp)
+        );
+        assert_eq!(
+            CompressionMethod::try_from(8),
+            Ok(CompressionMethod::NameTokenizer)
+        );
+        assert_eq!(CompressionMethod::try_from(9), Err(TryFromByteError(9)));
     }
 
     #[test]
@@ -93,6 +128,10 @@ mod tests {
         assert_eq!(u8::from(CompressionMethod::Gzip), 1);
         assert_eq!(u8::from(CompressionMethod::Bzip2), 2);
         assert_eq!(u8::from(CompressionMethod::Lzma), 3);
-        assert_eq!(u8::from(CompressionMethod::Rans), 4);
+        assert_eq!(u8::from(CompressionMethod::Rans4x8), 4);
+        assert_eq!(u8::from(CompressionMethod::RansNx16), 5);
+        assert_eq!(u8::from(CompressionMethod::AdaptiveArithmeticCoding), 6);
+        assert_eq!(u8::from(CompressionMethod::Fqzcomp), 7);
+        assert_eq!(u8::from(CompressionMethod::NameTokenizer), 8);
     }
 }
