@@ -2,29 +2,27 @@ mod builder;
 
 pub use self::builder::Builder;
 
-use noodles_sam as sam;
-
-use crate::num::{Itf8, Ltf8};
+use noodles_core::Position;
 
 use super::ReferenceSequenceId;
 
 // § 9 End of file container (2020-06-22)
-const EOF_LEN: i32 = 15;
-const EOF_START_POSITION: Itf8 = 4_542_278;
+const EOF_LEN: usize = 15;
+const EOF_START_POSITION: usize = 4_542_278;
 const EOF_BLOCK_COUNT: usize = 1;
 const EOF_CRC32: u32 = 0x4f_d9_bd_05;
 
 #[derive(Clone, Debug, Default, Eq, PartialEq)]
 pub struct Header {
-    length: i32,
+    length: usize,
     reference_sequence_id: ReferenceSequenceId,
-    start_position: Option<sam::record::Position>,
-    alignment_span: Itf8,
-    record_count: Itf8,
-    record_counter: Ltf8,
-    base_count: Ltf8,
+    start_position: Option<Position>,
+    alignment_span: usize,
+    record_count: i32,
+    record_counter: i64,
+    base_count: i64,
     block_count: usize,
-    landmarks: Vec<Itf8>,
+    landmarks: Vec<usize>,
     crc32: u32,
 }
 
@@ -38,14 +36,14 @@ impl Header {
     pub fn eof() -> Self {
         Self {
             length: EOF_LEN,
-            start_position: Some(sam::record::Position::try_from(EOF_START_POSITION).unwrap()),
+            start_position: Position::new(EOF_START_POSITION),
             block_count: EOF_BLOCK_COUNT,
             crc32: EOF_CRC32,
             ..Default::default()
         }
     }
 
-    pub fn len(&self) -> i32 {
+    pub fn len(&self) -> usize {
         self.length
     }
 
@@ -53,23 +51,23 @@ impl Header {
         self.reference_sequence_id
     }
 
-    pub fn start_position(&self) -> Option<sam::record::Position> {
+    pub fn start_position(&self) -> Option<Position> {
         self.start_position
     }
 
-    pub fn alignment_span(&self) -> Itf8 {
+    pub fn alignment_span(&self) -> usize {
         self.alignment_span
     }
 
-    pub fn record_count(&self) -> Itf8 {
+    pub fn record_count(&self) -> i32 {
         self.record_count
     }
 
-    pub fn record_counter(&self) -> Ltf8 {
+    pub fn record_counter(&self) -> i64 {
         self.record_counter
     }
 
-    pub fn base_count(&self) -> Ltf8 {
+    pub fn base_count(&self) -> i64 {
         self.base_count
     }
 
@@ -77,7 +75,7 @@ impl Header {
         self.block_count
     }
 
-    pub fn landmarks(&self) -> &[Itf8] {
+    pub fn landmarks(&self) -> &[usize] {
         &self.landmarks
     }
 
@@ -86,7 +84,7 @@ impl Header {
             && self.reference_sequence_id.is_none()
             && self
                 .start_position
-                .map(|position| i32::from(position) == EOF_START_POSITION)
+                .map(|position| usize::from(position) == EOF_START_POSITION)
                 .unwrap_or(false)
             && self.alignment_span == 0
             && self.record_count == 0
