@@ -101,7 +101,7 @@ where
 
 #[cfg(test)]
 mod tests {
-    use noodles_bam as bam;
+    use noodles_core::Position;
 
     use super::*;
     use crate::crai::Record;
@@ -114,8 +114,8 @@ mod tests {
         let actual = read_index(&mut reader).await?;
 
         let expected = vec![Record::new(
-            bam::record::ReferenceSequenceId::try_from(0).map(Some)?,
-            10946,
+            Some(0),
+            Position::new(10946),
             6765,
             17711,
             233,
