@@ -1,15 +1,15 @@
 use std::{io, mem};
 
 use noodles_fasta as fasta;
+use noodles_sam as sam;
 
-use super::{compression_header, slice, CompressionHeader, DataContainer, Slice};
-use crate::Record;
+use super::{slice, CompressionHeader, DataContainer, Slice};
+use crate::{writer::Options, Record};
 
-const MAX_SLICE_COUNT: usize = 4;
+const MAX_SLICE_COUNT: usize = 1;
 
 #[derive(Debug)]
 pub struct Builder {
-    compression_header_builder: compression_header::Builder,
     slice_builder: slice::Builder,
     slice_builders: Vec<slice::Builder>,
     record_counter: i64,
@@ -25,7 +25,6 @@ pub enum AddRecordError {
 impl Builder {
     pub fn new(record_counter: i64) -> Self {
         Self {
-            compression_header_builder: CompressionHeader::builder(),
             slice_builder: Slice::builder(),
             slice_builders: Vec::new(),
             record_counter,
@@ -41,22 +40,14 @@ impl Builder {
         self.base_count
     }
 
-    pub fn add_record(
-        &mut self,
-        reference_sequence: &[u8],
-        record: Record,
-    ) -> Result<(), AddRecordError> {
+    pub fn add_record(&mut self, record: Record) -> Result<(), AddRecordError> {
         if self.slice_builders.len() >= MAX_SLICE_COUNT {
             return Err(AddRecordError::ContainerFull(record));
         }
 
         match self.slice_builder.add_record(record) {
             Ok(r) => {
-                self.compression_header_builder
-                    .update(reference_sequence, r);
-
                 self.base_count += r.read_length() as i64;
-
                 Ok(())
             }
             Err(e) => match e {
@@ -65,25 +56,34 @@ impl Builder {
                     self.slice_builders.push(slice_builder);
                     Err(AddRecordError::SliceFull(r))
                 }
-                slice::builder::AddRecordError::ReferenceSequenceIdMismatch(r) => {
-                    Err(AddRecordError::ContainerFull(r))
-                }
             },
         }
     }
 
-    pub fn build(mut self, reference_sequences: &[fasta::Record]) -> io::Result<DataContainer> {
+    pub fn build(
+        mut self,
+        options: &Options,
+        reference_sequence_repository: &fasta::Repository,
+        header: &sam::Header,
+    ) -> io::Result<DataContainer> {
         if !self.slice_builder.is_empty() {
             self.slice_builders.push(self.slice_builder);
         }
 
-        let compression_header = self.compression_header_builder.build();
+        let compression_header = build_compression_header(options, &self.slice_builders);
 
         let record_counter = self.record_counter;
         let slices = self
             .slice_builders
             .into_iter()
-            .map(|builder| builder.build(reference_sequences, &compression_header, record_counter))
+            .map(|builder| {
+                builder.build(
+                    reference_sequence_repository,
+                    header,
+                    &compression_header,
+                    record_counter,
+                )
+            })
             .collect::<Result<_, _>>()?;
 
         Ok(DataContainer {
@@ -92,3 +92,19 @@ impl Builder {
         })
     }
 }
+
+fn build_compression_header(
+    options: &Options,
+    slice_builders: &[slice::Builder],
+) -> CompressionHeader {
+    let mut compression_header_builder = CompressionHeader::builder();
+    compression_header_builder.apply_options(options);
+
+    for slice_builder in slice_builders {
+        for record in slice_builder.records() {
+            compression_header_builder.update(record);
+        }
+    }
+
+    compression_header_builder.build()
+}
