@@ -1,9 +1,8 @@
-use crate::Record;
-
 use super::{
     data_series_encoding_map::DataSeriesEncodingMap, preservation_map, tag_encoding_map,
     CompressionHeader,
 };
+use crate::{writer::Options, Record};
 
 #[derive(Debug, Default)]
 pub struct Builder {
@@ -12,9 +11,12 @@ pub struct Builder {
 }
 
 impl Builder {
-    pub fn update(&mut self, reference_sequence: &[u8], record: &Record) {
-        self.preservation_map_builder
-            .update(reference_sequence, record);
+    pub fn apply_options(&mut self, options: &Options) {
+        self.preservation_map_builder.apply_options(options);
+    }
+
+    pub fn update(&mut self, record: &Record) {
+        self.preservation_map_builder.update(record);
         self.tag_encoding_map_builder.update(record);
     }
 
