@@ -1,12 +1,12 @@
-mod base;
 mod builder;
 mod histogram;
 
-pub use self::{base::Base, builder::Builder};
+pub use self::builder::Builder;
 
-use std::{error, fmt};
+use std::{cmp, error, fmt};
 
 use self::histogram::Histogram;
+use crate::record::feature::substitution::Base;
 
 type Substitutions = [[Base; 4]; 5];
 
@@ -23,6 +23,16 @@ impl SubstitutionMatrix {
     pub fn get(&self, reference_base: Base, substitution_code: u8) -> Base {
         self.substitutions[reference_base as usize][substitution_code as usize]
     }
+
+    pub fn find_code(&self, reference_base: Base, read_base: Base) -> u8 {
+        for code in [0b00, 0b01, 0b10, 0b11] {
+            if self.get(reference_base, code) == read_base {
+                return code;
+            }
+        }
+
+        unreachable!();
+    }
 }
 
 impl Default for SubstitutionMatrix {
@@ -41,17 +51,21 @@ impl Default for SubstitutionMatrix {
 
 impl From<Histogram> for SubstitutionMatrix {
     fn from(histogram: Histogram) -> Self {
-        let mut matrix = Self::default();
-        let bases = [Base::A, Base::C, Base::G, Base::T, Base::N];
+        const BASES: [Base; 5] = [Base::A, Base::C, Base::G, Base::T, Base::N];
 
-        for &reference_base in bases.iter() {
-            let mut base_frequency_pairs: Vec<_> = bases
+        let mut matrix = Self::default();
+
+        for reference_base in BASES {
+            let mut base_frequency_pairs: Vec<_> = BASES
                 .iter()
                 .zip(histogram.get_bins(reference_base).iter())
                 .filter(|(&read_base, _)| read_base != reference_base)
                 .collect();
 
-            base_frequency_pairs.sort_by_key(|(_, &frequency)| -(frequency as i64));
+            // § 10.6 "Mapped reads" (2021-10-15): "the substitutions for each reference base may
+            // optionally be sorted by their frequencies, in descending order, with same-frequency
+            // ties broken using the fixed order ACGTN."
+            base_frequency_pairs.sort_by_key(|(&base, &frequency)| (cmp::Reverse(frequency), base));
 
             for (code, (&read_base, _)) in base_frequency_pairs.iter().enumerate() {
                 let i = reference_base as usize;
@@ -164,6 +178,24 @@ impl From<&SubstitutionMatrix> for [u8; 5] {
 mod tests {
     use super::*;
 
+    #[test]
+    fn test_find_code() {
+        let matrix = SubstitutionMatrix {
+            substitutions: [
+                [Base::T, Base::G, Base::C, Base::N],
+                [Base::A, Base::G, Base::T, Base::N],
+                [Base::N, Base::A, Base::C, Base::T],
+                [Base::G, Base::N, Base::A, Base::C],
+                [Base::C, Base::G, Base::T, Base::A],
+            ],
+        };
+
+        assert_eq!(matrix.find_code(Base::A, Base::T), 0b00);
+        assert_eq!(matrix.find_code(Base::C, Base::G), 0b01);
+        assert_eq!(matrix.find_code(Base::G, Base::C), 0b10);
+        assert_eq!(matrix.find_code(Base::T, Base::C), 0b11);
+    }
+
     #[test]
     fn test_try_from_u8_slice() -> Result<(), TryFromByteArrayError> {
         let codes = [0x93, 0x1b, 0x6c, 0xb1, 0xc6];
@@ -203,7 +235,7 @@ mod tests {
     fn test_from_histogram() {
         let histogram = Histogram::new([
             [0, 3, 8, 5, 0],
-            [0, 0, 0, 0, 0],
+            [2, 0, 5, 5, 2],
             [0, 0, 0, 0, 0],
             [0, 0, 0, 0, 0],
             [0, 0, 0, 0, 0],
@@ -213,7 +245,7 @@ mod tests {
 
         let expected = [
             [Base::G, Base::T, Base::C, Base::N],
-            [Base::A, Base::G, Base::T, Base::N],
+            [Base::G, Base::T, Base::A, Base::N],
             [Base::A, Base::C, Base::T, Base::N],
             [Base::A, Base::C, Base::G, Base::N],
             [Base::A, Base::C, Base::G, Base::T],
