@@ -1,28 +1,21 @@
 mod builder;
-mod embedded_reference_bases_block_content_id;
 
-pub use {
-    builder::Builder,
-    embedded_reference_bases_block_content_id::EmbeddedReferenceBasesBlockContentId,
-};
+pub use builder::Builder;
 
-use noodles_sam as sam;
+use noodles_core::Position;
 
-use crate::{
-    container::ReferenceSequenceId,
-    num::{Itf8, Ltf8},
-};
+use crate::container::ReferenceSequenceId;
 
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct Header {
     reference_sequence_id: ReferenceSequenceId,
-    alignment_start: Option<sam::record::Position>,
-    alignment_span: Itf8,
+    alignment_start: Option<Position>,
+    alignment_span: usize,
     record_count: usize,
-    record_counter: Ltf8,
+    record_counter: i64,
     block_count: usize,
-    block_content_ids: Vec<Itf8>,
-    embedded_reference_bases_block_content_id: EmbeddedReferenceBasesBlockContentId,
+    block_content_ids: Vec<i32>,
+    embedded_reference_bases_block_content_id: Option<i32>,
     reference_md5: [u8; 16],
     optional_tags: Vec<u8>,
 }
@@ -36,19 +29,26 @@ impl Header {
         self.reference_sequence_id
     }
 
-    pub fn alignment_start(&self) -> Option<sam::record::Position> {
+    pub fn alignment_start(&self) -> Option<Position> {
         self.alignment_start
     }
 
-    pub fn alignment_span(&self) -> Itf8 {
+    pub fn alignment_span(&self) -> usize {
         self.alignment_span
     }
 
+    pub fn alignment_end(&self) -> Option<Position> {
+        self.alignment_start().and_then(|start| {
+            let end = usize::from(start) + self.alignment_span() + 1;
+            Position::new(end)
+        })
+    }
+
     pub fn record_count(&self) -> usize {
         self.record_count
     }
 
-    pub fn record_counter(&self) -> Ltf8 {
+    pub fn record_counter(&self) -> i64 {
         self.record_counter
     }
 
@@ -56,13 +56,11 @@ impl Header {
         self.block_count
     }
 
-    pub fn block_content_ids(&self) -> &[Itf8] {
+    pub fn block_content_ids(&self) -> &[i32] {
         &self.block_content_ids
     }
 
-    pub fn embedded_reference_bases_block_content_id(
-        &self,
-    ) -> EmbeddedReferenceBasesBlockContentId {
+    pub fn embedded_reference_bases_block_content_id(&self) -> Option<i32> {
         self.embedded_reference_bases_block_content_id
     }
 
