@@ -1,22 +1,18 @@
-use noodles_sam as sam;
+use noodles_core::Position;
 
-use crate::{
-    container::ReferenceSequenceId,
-    num::{Itf8, Ltf8},
-};
-
-use super::{EmbeddedReferenceBasesBlockContentId, Header};
+use super::Header;
+use crate::container::ReferenceSequenceId;
 
 #[derive(Default)]
 pub struct Builder {
     reference_sequence_id: ReferenceSequenceId,
-    alignment_start: Option<sam::record::Position>,
-    alignment_span: Itf8,
+    alignment_start: Option<Position>,
+    alignment_span: usize,
     record_count: usize,
-    record_counter: Ltf8,
+    record_counter: i64,
     block_count: usize,
-    block_content_ids: Vec<Itf8>,
-    embedded_reference_bases_block_content_id: EmbeddedReferenceBasesBlockContentId,
+    block_content_ids: Vec<i32>,
+    embedded_reference_bases_block_content_id: Option<i32>,
     reference_md5: [u8; 16],
     optional_tags: Vec<u8>,
 }
@@ -27,12 +23,12 @@ impl Builder {
         self
     }
 
-    pub fn set_alignment_start(mut self, alignment_start: sam::record::Position) -> Self {
+    pub fn set_alignment_start(mut self, alignment_start: Position) -> Self {
         self.alignment_start = Some(alignment_start);
         self
     }
 
-    pub fn set_alignment_span(mut self, alignment_span: Itf8) -> Self {
+    pub fn set_alignment_span(mut self, alignment_span: usize) -> Self {
         self.alignment_span = alignment_span;
         self
     }
@@ -42,7 +38,7 @@ impl Builder {
         self
     }
 
-    pub fn set_record_counter(mut self, record_counter: Ltf8) -> Self {
+    pub fn set_record_counter(mut self, record_counter: i64) -> Self {
         self.record_counter = record_counter;
         self
     }
@@ -52,16 +48,13 @@ impl Builder {
         self
     }
 
-    pub fn set_block_content_ids(mut self, block_content_ids: Vec<Itf8>) -> Self {
+    pub fn set_block_content_ids(mut self, block_content_ids: Vec<i32>) -> Self {
         self.block_content_ids = block_content_ids;
         self
     }
 
-    pub fn set_embedded_reference_bases_block_content_id(
-        mut self,
-        embedded_reference_bases_block_content_id: EmbeddedReferenceBasesBlockContentId,
-    ) -> Self {
-        self.embedded_reference_bases_block_content_id = embedded_reference_bases_block_content_id;
+    pub fn set_embedded_reference_bases_block_content_id(mut self, id: i32) -> Self {
+        self.embedded_reference_bases_block_content_id = Some(id);
         self
     }
 
