@@ -1,13 +1,12 @@
 //! CRAM reader and record iterator.
 
-mod container;
+pub(crate) mod container;
 pub(crate) mod data_container;
 pub(crate) mod num;
+mod query;
 pub(crate) mod record;
 mod records;
 
-use crate::data_container::DataContainer;
-
 pub use self::records::Records;
 
 use std::{
@@ -16,9 +15,15 @@ use std::{
 };
 
 use byteorder::{LittleEndian, ReadBytesExt};
+use bytes::BytesMut;
+use noodles_core::{region::Interval, Region};
+use noodles_fasta as fasta;
+use noodles_sam as sam;
 
 use self::container::read_container;
-use super::{container::Block, file_definition::Version, FileDefinition, MAGIC_NUMBER};
+pub use self::query::Query;
+use super::{container::Block, crai, file_definition::Version, FileDefinition, MAGIC_NUMBER};
+use crate::data_container::DataContainer;
 
 /// A CRAM reader.
 ///
@@ -30,23 +35,28 @@ use super::{container::Block, file_definition::Version, FileDefinition, MAGIC_NU
 /// ```no_run
 /// # use std::{fs::File, io};
 /// use noodles_cram as cram;
+/// use noodles_fasta as fasta;
+///
+/// let repository = fasta::Repository::default();
 ///
 /// let mut reader = File::open("sample.cram").map(cram::Reader::new)?;
 /// reader.read_file_definition()?;
-/// reader.read_file_header()?;
 ///
-/// for result in reader.records() {
+/// let header = reader.read_file_header()?.parse()?;
+///
+/// for result in reader.records(&repository, &header) {
 ///     let record = result?;
 ///     println!("{:?}", record);
 /// }
 ///
-/// # Ok::<(), io::Error>(())
+/// # Ok::<_, Box<dyn std::error::Error>>(())
 /// ```
 pub struct Reader<R>
 where
     R: Read,
 {
     inner: R,
+    buf: BytesMut,
 }
 
 impl<R> Reader<R>
@@ -64,7 +74,10 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn new(reader: R) -> Self {
-        Self { inner: reader }
+        Self {
+            inner: reader,
+            buf: BytesMut::new(),
+        }
     }
 
     /// Returns a reference to the underlying reader.
@@ -154,7 +167,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_file_header(&mut self) -> io::Result<String> {
-        let container = read_container(&mut self.inner)?;
+        let container = read_container(&mut self.inner, &mut self.buf)?;
 
         if let Some(block) = container.blocks().first() {
             read_file_header_block(block)
@@ -170,7 +183,7 @@ where
         &mut self,
     ) -> io::Result<Option<(crate::container::Header, DataContainer)>> {
         use self::data_container::read_data_container_with_container_header;
-        read_data_container_with_container_header(&mut self.inner)
+        read_data_container_with_container_header(&mut self.inner, &mut self.buf)
     }
 
     /// Reads a data container.
@@ -196,7 +209,7 @@ where
     pub fn read_data_container(&mut self) -> io::Result<Option<DataContainer>> {
         use self::data_container::read_data_container;
 
-        read_data_container(&mut self.inner)
+        read_data_container(&mut self.inner, &mut self.buf)
     }
 
     /// Returns a iterator over records starting from the current stream position.
@@ -208,19 +221,27 @@ where
     /// ```no_run
     /// # use std::{fs::File, io};
     /// use noodles_cram as cram;
+    /// use noodles_fasta as fasta;
+    ///
+    /// let repository = fasta::Repository::default();
     ///
     /// let mut reader = File::open("sample.cram").map(cram::Reader::new)?;
     /// reader.read_file_definition()?;
-    /// reader.read_file_header()?;
     ///
-    /// for result in reader.records() {
+    /// let header = reader.read_file_header()?.parse()?;
+    ///
+    /// for result in reader.records(&repository, &header) {
     ///     let record = result?;
     ///     println!("{:?}", record);
     /// }
-    /// # Ok::<(), io::Error>(())
+    /// # Ok::<_, Box<dyn std::error::Error>>(())
     /// ```
-    pub fn records(&mut self) -> Records<'_, R> {
-        Records::new(self)
+    pub fn records<'a>(
+        &'a mut self,
+        reference_sequence_repository: &'a fasta::Repository,
+        header: &'a sam::Header,
+    ) -> Records<'a, R> {
+        Records::new(self, reference_sequence_repository, header)
     }
 }
 
@@ -263,6 +284,84 @@ where
     pub fn position(&mut self) -> io::Result<u64> {
         self.inner.seek(SeekFrom::Current(0))
     }
+
+    /// Returns an iterator over records that intersects the given region.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use std::{fs::File, io};
+    /// use noodles_cram::{self as cram, crai};
+    /// use noodles_fasta as fasta;
+    ///
+    /// let mut reader = File::open("sample.cram").map(cram::Reader::new)?;
+    /// reader.read_file_definition()?;
+    ///
+    /// let repository = fasta::Repository::default();
+    /// let header = reader.read_file_header()?.parse()?;
+    /// let index = crai::read("sample.cram.crai")?;
+    /// let region = "sq0:8-13".parse()?;
+    /// let query = reader.query(&repository, &header, &index, &region)?;
+    ///
+    /// for result in query {
+    ///     let record = result?;
+    ///     // ...
+    /// }
+    /// # Ok::<_, Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn query<'a>(
+        &'a mut self,
+        reference_sequence_repository: &'a fasta::Repository,
+        header: &'a sam::Header,
+        index: &'a crai::Index,
+        region: &Region,
+    ) -> io::Result<Query<'_, R, Interval>> {
+        let reference_sequence_id = header
+            .reference_sequences()
+            .get_index_of(region.name())
+            .ok_or_else(|| {
+                io::Error::new(
+                    io::ErrorKind::InvalidInput,
+                    "invalid reference sequence name",
+                )
+            })?;
+
+        Ok(Query::new(
+            self,
+            reference_sequence_repository,
+            header,
+            index,
+            reference_sequence_id,
+            region.interval(),
+        ))
+    }
+}
+
+impl<R> sam::AlignmentReader for Reader<R>
+where
+    R: Read,
+{
+    fn read_alignment_header(&mut self) -> io::Result<sam::Header> {
+        self.read_file_definition()?;
+
+        self.read_file_header().and_then(|s| {
+            s.parse()
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+        })
+    }
+
+    fn alignment_records<'a>(
+        &'a mut self,
+        reference_sequence_repository: &'a fasta::Repository,
+        header: &'a sam::Header,
+    ) -> Box<dyn Iterator<Item = io::Result<Box<dyn sam::AlignmentRecord>>> + 'a> {
+        Box::new(
+            self.records(reference_sequence_repository, header)
+                .map(|result| {
+                    result.map(|record| Box::new(record) as Box<dyn sam::AlignmentRecord>)
+                }),
+        )
+    }
 }
 
 fn read_magic_number<R>(reader: &mut R) -> io::Result<()>
@@ -352,18 +451,21 @@ mod tests {
 
     #[test]
     fn test_read_file_header_block() -> io::Result<()> {
+        use bytes::BufMut;
+
         let expected = "noodles";
 
         let header_data = expected.as_bytes();
         let header_data_len = header_data.len() as i32;
 
-        let mut data = header_data_len.to_le_bytes().to_vec();
-        data.extend(header_data);
+        let mut data = BytesMut::new();
+        data.put_i32_le(header_data_len);
+        data.extend_from_slice(header_data);
 
         let block = Block::builder()
             .set_content_type(ContentType::FileHeader)
             .set_uncompressed_len(data.len())
-            .set_data(data.to_vec())
+            .set_data(data.freeze())
             .build();
 
         let actual = read_file_header_block(&block)?;
