@@ -2,25 +2,32 @@ mod external_data_readers;
 
 pub use external_data_readers::ExternalDataReaders;
 
-use std::{
-    error, fmt,
-    io::{self, BufRead, Read},
+use std::{error, fmt, io};
+
+use bytes::Buf;
+use noodles_bam as bam;
+use noodles_core::Position;
+use noodles_sam::{
+    self as sam,
+    record::{quality_scores::Score, sequence::Base},
+    AlignmentRecord,
 };
 
-use byteorder::ReadBytesExt;
-use noodles_bam as bam;
-use noodles_sam as sam;
-
-use super::num::read_itf8;
+use super::num::get_itf8;
 use crate::{
     container::ReferenceSequenceId,
     data_container::{
-        compression_header::{data_series_encoding_map::DataSeries, encoding::Encoding},
+        compression_header::{
+            data_series_encoding_map::DataSeries, encoding::Encoding,
+            preservation_map::tag_ids_dictionary,
+        },
         CompressionHeader,
     },
     huffman::CanonicalHuffmanDecoder,
-    num::Itf8,
-    record::{feature, tag, Feature, Flags, NextMateFlags, ReadGroupId, Tag},
+    record::{
+        feature::{self, substitution},
+        Feature, Flags, NextMateFlags,
+    },
     BitReader, Record,
 };
 
@@ -28,7 +35,7 @@ use crate::{
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub enum ReadRecordError {
     MissingDataSeriesEncoding(DataSeries),
-    MissingTagEncoding(tag::Key),
+    MissingTagEncoding(tag_ids_dictionary::Key),
     MissingExternalBlock(i32),
 }
 
@@ -50,27 +57,27 @@ impl fmt::Display for ReadRecordError {
 
 pub struct Reader<'a, CDR, EDR>
 where
-    CDR: Read,
-    EDR: BufRead,
+    CDR: Buf,
+    EDR: Buf,
 {
     compression_header: &'a CompressionHeader,
     core_data_reader: BitReader<CDR>,
     external_data_readers: ExternalDataReaders<EDR>,
     reference_sequence_id: ReferenceSequenceId,
-    prev_alignment_start: Option<sam::record::Position>,
+    prev_alignment_start: Option<Position>,
 }
 
 impl<'a, CDR, EDR> Reader<'a, CDR, EDR>
 where
-    CDR: Read,
-    EDR: BufRead,
+    CDR: Buf,
+    EDR: Buf,
 {
     pub fn new(
         compression_header: &'a CompressionHeader,
         core_data_reader: BitReader<CDR>,
         external_data_readers: ExternalDataReaders<EDR>,
         reference_sequence_id: ReferenceSequenceId,
-        initial_alignment_start: Option<sam::record::Position>,
+        initial_alignment_start: Option<Position>,
     ) -> Self {
         Self {
             compression_header,
@@ -139,21 +146,12 @@ where
     }
 
     fn read_positional_data(&mut self, record: &mut Record) -> io::Result<usize> {
-        let reference_id = if self.reference_sequence_id.is_many() {
-            self.read_reference_id()?
-        } else {
-            i32::from(self.reference_sequence_id)
+        record.reference_sequence_id = match self.reference_sequence_id {
+            ReferenceSequenceId::Some(id) => Some(id),
+            ReferenceSequenceId::None => None,
+            ReferenceSequenceId::Many => self.read_reference_id()?,
         };
 
-        record.reference_sequence_id =
-            if reference_id == bam::record::reference_sequence_id::UNMAPPED {
-                None
-            } else {
-                bam::record::ReferenceSequenceId::try_from(reference_id)
-                    .map(Some)
-                    .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?
-            };
-
         let read_length = self.read_read_length()?;
         record.read_length = read_length;
 
@@ -163,8 +161,11 @@ where
         Ok(read_length)
     }
 
-    fn read_reference_id(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_reference_id(&mut self) -> io::Result<Option<usize>> {
+        use bam::record::reference_sequence_id::UNMAPPED;
+
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .reference_id_encoding()
             .ok_or_else(|| {
@@ -172,14 +173,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::ReferenceId),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| match n {
+            UNMAPPED => Ok(None),
+            _ => usize::try_from(n)
+                .map(Some)
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)),
+        })
     }
 
     fn read_read_length(&mut self) -> io::Result<usize> {
@@ -196,7 +202,7 @@ where
         .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_alignment_start(&mut self) -> io::Result<Option<sam::record::Position>> {
+    fn read_alignment_start(&mut self) -> io::Result<Option<Position>> {
         let ap_data_series_delta = self
             .compression_header
             .preservation_map()
@@ -207,28 +213,34 @@ where
             .data_series_encoding_map()
             .in_seq_positions_encoding();
 
-        let mut alignment_start = decode_itf8(
+        let alignment_start_or_delta = decode_itf8(
             encoding,
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )?;
 
-        if ap_data_series_delta {
-            let prev_alignment_start = self.prev_alignment_start.map(i32::from).unwrap_or_default();
-            let delta = alignment_start;
-            alignment_start = prev_alignment_start + delta;
-        }
+        let alignment_start = if ap_data_series_delta {
+            let prev_alignment_start = i32::try_from(
+                self.prev_alignment_start
+                    .map(usize::from)
+                    .unwrap_or_default(),
+            )
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
 
-        if alignment_start == 0 {
-            Ok(None)
+            prev_alignment_start + alignment_start_or_delta
         } else {
-            sam::record::Position::try_from(alignment_start)
-                .map(Some)
-                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-        }
+            alignment_start_or_delta
+        };
+
+        usize::try_from(alignment_start)
+            .map(Position::new)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
     }
 
-    fn read_read_group(&mut self) -> io::Result<ReadGroupId> {
+    fn read_read_group(&mut self) -> io::Result<Option<usize>> {
+        // § 10.2 "CRAM positional data" (2021-10-15): "-1 for no group".
+        const MISSING: i32 = -1;
+
         let encoding = self
             .compression_header
             .data_series_encoding_map()
@@ -239,7 +251,12 @@ where
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )
-        .map(ReadGroupId::from)
+        .and_then(|n| match n {
+            MISSING => Ok(None),
+            _ => usize::try_from(n)
+                .map(Some)
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)),
+        })
     }
 
     fn read_read_names(&mut self, record: &mut Record) -> io::Result<()> {
@@ -253,8 +270,11 @@ where
         Ok(())
     }
 
-    fn read_read_name(&mut self) -> io::Result<Vec<u8>> {
-        self.compression_header
+    fn read_read_name(&mut self) -> io::Result<Option<sam::record::ReadName>> {
+        use sam::record::read_name::MISSING;
+
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .read_names_encoding()
             .ok_or_else(|| {
@@ -262,15 +282,21 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::ReadNames),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte_array(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                    None,
-                )
-            })
+            })?;
+
+        let buf = decode_byte_array(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+            None,
+        )?;
+
+        match &buf[..] {
+            MISSING => Ok(None),
+            _ => sam::record::ReadName::try_from(buf)
+                .map(Some)
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)),
+        }
     }
 
     fn read_mate_data(
@@ -305,7 +331,7 @@ where
             record.next_mate_alignment_start = self.read_next_mate_alignment_start()?;
             record.template_size = self.read_template_size()?;
         } else if flags.has_mate_downstream() {
-            record.distance_to_next_fragment = self.read_distance_to_next_fragment()?;
+            record.distance_to_next_fragment = self.read_distance_to_next_fragment().map(Some)?;
         }
 
         Ok(())
@@ -332,9 +358,9 @@ where
         .map(NextMateFlags::from)
     }
 
-    fn read_next_fragment_reference_sequence_id(
-        &mut self,
-    ) -> io::Result<Option<bam::record::ReferenceSequenceId>> {
+    fn read_next_fragment_reference_sequence_id(&mut self) -> io::Result<Option<usize>> {
+        use bam::record::reference_sequence_id::UNMAPPED;
+
         let encoding = self
             .compression_header
             .data_series_encoding_map()
@@ -353,18 +379,15 @@ where
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )
-        .and_then(|id| {
-            if id == bam::record::reference_sequence_id::UNMAPPED {
-                Ok(None)
-            } else {
-                bam::record::ReferenceSequenceId::try_from(id)
-                    .map(Some)
-                    .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-            }
+        .and_then(|id| match id {
+            UNMAPPED => Ok(None),
+            _ => usize::try_from(id)
+                .map(Some)
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)),
         })
     }
 
-    fn read_next_mate_alignment_start(&mut self) -> io::Result<Option<sam::record::Position>> {
+    fn read_next_mate_alignment_start(&mut self) -> io::Result<Option<Position>> {
         let encoding = self
             .compression_header
             .data_series_encoding_map()
@@ -381,18 +404,11 @@ where
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )
-        .and_then(|n| {
-            if n == 0 {
-                Ok(None)
-            } else {
-                sam::record::Position::try_from(n)
-                    .map(Some)
-                    .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-            }
-        })
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+        .map(Position::new)
     }
 
-    fn read_template_size(&mut self) -> io::Result<Itf8> {
+    fn read_template_size(&mut self) -> io::Result<i32> {
         self.compression_header
             .data_series_encoding_map()
             .template_size_encoding()
@@ -411,8 +427,9 @@ where
             })
     }
 
-    fn read_distance_to_next_fragment(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_distance_to_next_fragment(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .distance_to_next_fragment_encoding()
             .ok_or_else(|| {
@@ -420,22 +437,21 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::DistanceToNextFragment),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_tag_data(&mut self) -> io::Result<Vec<Tag>> {
-        use bam::reader::record::data::field::read_value;
+    fn read_tag_data(&mut self) -> io::Result<sam::record::Data> {
+        use bam::reader::record::data::field::get_value;
+        use sam::record::data::Field;
 
-        let tag_line = self.read_tag_line().and_then(|i| {
-            usize::try_from(i).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-        })?;
+        let tag_line = self.read_tag_line()?;
 
         let tag_keys = self
             .compression_header
@@ -446,7 +462,7 @@ where
 
         let tag_encoding_map = self.compression_header.tag_encoding_map();
 
-        let mut tags = Vec::with_capacity(tag_keys.len());
+        let mut fields = Vec::with_capacity(tag_keys.len());
 
         for key in tag_keys {
             let id = key.id();
@@ -465,16 +481,17 @@ where
             )?;
 
             let mut data_reader = &data[..];
-            let value = read_value(&mut data_reader, key.ty())?;
+            let value = get_value(&mut data_reader, key.ty())?;
 
-            let tag = Tag::new(*key, value);
-            tags.push(tag);
+            let field = Field::new(key.tag(), value);
+            fields.push(field);
         }
 
-        Ok(tags)
+        sam::record::Data::try_from(fields)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
     }
 
-    fn read_tag_line(&mut self) -> io::Result<Itf8> {
+    fn read_tag_line(&mut self) -> io::Result<usize> {
         let encoding = self
             .compression_header
             .data_series_encoding_map()
@@ -485,6 +502,7 @@ where
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
     fn read_mapped_read(
@@ -499,13 +517,15 @@ where
 
         for _ in 0..feature_count {
             let feature = self.read_feature(prev_position)?;
-            prev_position = feature.position();
+            prev_position = usize::from(feature.position());
             record.add_feature(feature);
         }
 
         record.mapping_quality = self.read_mapping_quality()?;
 
         if flags.are_quality_scores_stored_as_array() {
+            record.quality_scores.as_mut().reserve(read_length);
+
             for _ in 0..read_length {
                 let score = self.read_quality_score()?;
                 record.quality_scores.push(score);
@@ -515,8 +535,9 @@ where
         Ok(())
     }
 
-    fn read_number_of_read_features(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_number_of_read_features(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .number_of_read_features_encoding()
             .ok_or_else(|| {
@@ -524,23 +545,24 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::NumberOfReadFeatures),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_feature(&mut self, prev_position: i32) -> io::Result<Feature> {
+    fn read_feature(&mut self, prev_position: usize) -> io::Result<Feature> {
         use feature::Code;
 
         let code = self.read_feature_code()?;
 
         let delta = self.read_feature_position()?;
-        let position = prev_position + delta;
+        let position = Position::try_from(prev_position + delta)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
 
         match code {
             Code::Bases => {
@@ -612,14 +634,14 @@ where
             &mut self.core_data_reader,
             &mut self.external_data_readers,
         )
-        .map(char::from)
         .and_then(|id| {
             feature::Code::try_from(id).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
         })
     }
 
-    fn read_feature_position(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_feature_position(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .in_read_positions_encoding()
             .ok_or_else(|| {
@@ -627,18 +649,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::InReadPositions),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_stretches_of_bases(&mut self) -> io::Result<Vec<u8>> {
-        self.compression_header
+    fn read_stretches_of_bases(&mut self) -> io::Result<Vec<Base>> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .stretches_of_bases_encoding()
             .ok_or_else(|| {
@@ -646,19 +669,24 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::StretchesOfBases),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte_array(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                    None,
-                )
-            })
+            })?;
+
+        let raw_bases = decode_byte_array(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+            None,
+        )?;
+
+        raw_bases
+            .into_iter()
+            .map(|n| Base::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+            .collect()
     }
 
-    fn read_stretches_of_quality_scores(&mut self) -> io::Result<Vec<u8>> {
-        self.compression_header
+    fn read_stretches_of_quality_scores(&mut self) -> io::Result<Vec<Score>> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .stretches_of_quality_scores_encoding()
             .ok_or_else(|| {
@@ -668,19 +696,24 @@ where
                         DataSeries::StretchesOfQualityScores,
                     ),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte_array(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                    None,
-                )
-            })
+            })?;
+
+        let scores = decode_byte_array(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+            None,
+        )?;
+
+        scores
+            .into_iter()
+            .map(|n| Score::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+            .collect()
     }
 
-    fn read_base(&mut self) -> io::Result<u8> {
-        self.compression_header
+    fn read_base(&mut self) -> io::Result<Base> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .bases_encoding()
             .ok_or_else(|| {
@@ -688,18 +721,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::Bases),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_byte(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| Base::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_quality_score(&mut self) -> io::Result<u8> {
-        self.compression_header
+    fn read_quality_score(&mut self) -> io::Result<Score> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .quality_scores_encoding()
             .ok_or_else(|| {
@@ -707,18 +741,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::QualityScores),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_byte(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| Score::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_base_substitution_code(&mut self) -> io::Result<u8> {
-        self.compression_header
+    fn read_base_substitution_code(&mut self) -> io::Result<substitution::Value> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .base_substitution_codes_encoding()
             .ok_or_else(|| {
@@ -726,18 +761,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::BaseSubstitutionCodes),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_byte(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .map(substitution::Value::Code)
     }
 
-    fn read_insertion(&mut self) -> io::Result<Vec<u8>> {
-        self.compression_header
+    fn read_insertion(&mut self) -> io::Result<Vec<Base>> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .insertion_encoding()
             .ok_or_else(|| {
@@ -745,19 +781,24 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::Insertion),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte_array(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                    None,
-                )
-            })
+            })?;
+
+        let raw_bases = decode_byte_array(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+            None,
+        )?;
+
+        raw_bases
+            .into_iter()
+            .map(|n| Base::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+            .collect()
     }
 
-    fn read_deletion_length(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_deletion_length(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .deletion_lengths_encoding()
             .ok_or_else(|| {
@@ -765,18 +806,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::DeletionLengths),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_reference_skip_length(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_reference_skip_length(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .reference_skip_length_encoding()
             .ok_or_else(|| {
@@ -784,18 +826,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::ReferenceSkipLength),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_soft_clip(&mut self) -> io::Result<Vec<u8>> {
-        self.compression_header
+    fn read_soft_clip(&mut self) -> io::Result<Vec<Base>> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .soft_clip_encoding()
             .ok_or_else(|| {
@@ -803,19 +846,24 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::SoftClip),
                 )
-            })
-            .and_then(|encoding| {
-                decode_byte_array(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                    None,
-                )
-            })
+            })?;
+
+        let raw_bases = decode_byte_array(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+            None,
+        )?;
+
+        raw_bases
+            .into_iter()
+            .map(|n| Base::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
+            .collect()
     }
 
-    fn read_padding(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_padding(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .padding_encoding()
             .ok_or_else(|| {
@@ -823,18 +871,19 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::Padding),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    fn read_hard_clip(&mut self) -> io::Result<Itf8> {
-        self.compression_header
+    fn read_hard_clip(&mut self) -> io::Result<usize> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .hard_clip_encoding()
             .ok_or_else(|| {
@@ -842,14 +891,14 @@ where
                     io::ErrorKind::InvalidData,
                     ReadRecordError::MissingDataSeriesEncoding(DataSeries::HardClip),
                 )
-            })
-            .and_then(|encoding| {
-                decode_itf8(
-                    encoding,
-                    &mut self.core_data_reader,
-                    &mut self.external_data_readers,
-                )
-            })
+            })?;
+
+        decode_itf8(
+            encoding,
+            &mut self.core_data_reader,
+            &mut self.external_data_readers,
+        )
+        .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
     fn read_mapping_quality(&mut self) -> io::Result<Option<sam::record::MappingQuality>> {
@@ -875,9 +924,7 @@ where
 
         match n {
             MISSING => Ok(None),
-            _ => sam::record::MappingQuality::try_from(n)
-                .map(Some)
-                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)),
+            _ => Ok(sam::record::MappingQuality::new(n)),
         }
     }
 
@@ -887,12 +934,16 @@ where
         flags: Flags,
         read_length: usize,
     ) -> io::Result<()> {
+        record.bases.as_mut().reserve(read_length);
+
         for _ in 0..read_length {
             let base = self.read_base()?;
             record.bases.push(base);
         }
 
         if flags.are_quality_scores_stored_as_array() {
+            record.quality_scores.as_mut().reserve(read_length);
+
             for _ in 0..read_length {
                 let score = self.read_quality_score()?;
                 record.quality_scores.push(score);
@@ -909,12 +960,12 @@ fn decode_byte<CDR, EDR>(
     external_data_readers: &mut ExternalDataReaders<EDR>,
 ) -> io::Result<u8>
 where
-    CDR: Read,
-    EDR: Read,
+    CDR: Buf,
+    EDR: Buf,
 {
     match encoding {
         Encoding::External(block_content_id) => {
-            let reader = external_data_readers
+            let src = external_data_readers
                 .get_mut(block_content_id)
                 .ok_or_else(|| {
                     io::Error::new(
@@ -923,7 +974,11 @@ where
                     )
                 })?;
 
-            reader.read_u8()
+            if !src.has_remaining() {
+                return Err(io::Error::from(io::ErrorKind::UnexpectedEof));
+            }
+
+            Ok(src.get_u8())
         }
         Encoding::Huffman(alphabet, bit_lens) => {
             if alphabet.len() == 1 {
@@ -933,9 +988,6 @@ where
                 decoder.decode(core_data_reader).map(|i| i as u8)
             }
         }
-        Encoding::Beta(offset, len) => core_data_reader
-            .read_u32(*len)
-            .map(|i| (i as i32 - offset) as u8),
         _ => todo!("decode_byte: {:?}", encoding),
     }
 }
@@ -944,14 +996,14 @@ fn decode_itf8<CDR, EDR>(
     encoding: &Encoding,
     core_data_reader: &mut BitReader<CDR>,
     external_data_readers: &mut ExternalDataReaders<EDR>,
-) -> io::Result<Itf8>
+) -> io::Result<i32>
 where
-    CDR: Read,
-    EDR: Read,
+    CDR: Buf,
+    EDR: Buf,
 {
     match encoding {
         Encoding::External(block_content_id) => {
-            let reader = external_data_readers
+            let src = external_data_readers
                 .get_mut(block_content_id)
                 .ok_or_else(|| {
                     io::Error::new(
@@ -960,7 +1012,7 @@ where
                     )
                 })?;
 
-            read_itf8(reader)
+            get_itf8(src)
         }
         Encoding::Huffman(alphabet, bit_lens) => {
             if alphabet.len() == 1 {
@@ -982,12 +1034,12 @@ fn decode_byte_array<CDR, EDR>(
     buf: Option<Vec<u8>>,
 ) -> io::Result<Vec<u8>>
 where
-    CDR: Read,
-    EDR: BufRead,
+    CDR: Buf,
+    EDR: Buf,
 {
     match encoding {
         Encoding::External(block_content_id) => {
-            let reader = external_data_readers
+            let src = external_data_readers
                 .get_mut(block_content_id)
                 .ok_or_else(|| {
                     io::Error::new(
@@ -996,8 +1048,13 @@ where
                     )
                 })?;
 
-            let mut buf = buf.expect("missing buf");
-            reader.read_exact(&mut buf)?;
+            let mut buf = buf.unwrap();
+
+            if src.remaining() < buf.len() {
+                return Err(io::Error::from(io::ErrorKind::UnexpectedEof));
+            }
+
+            src.copy_to_slice(&mut buf);
 
             Ok(buf)
         }
@@ -1015,7 +1072,7 @@ where
             Ok(value)
         }
         Encoding::ByteArrayStop(stop_byte, block_content_id) => {
-            let reader = external_data_readers
+            let src = external_data_readers
                 .get_mut(block_content_id)
                 .ok_or_else(|| {
                     io::Error::new(
@@ -1024,11 +1081,21 @@ where
                     )
                 })?;
 
-            let mut buf = Vec::new();
-            reader.read_until(*stop_byte, &mut buf)?;
+            let len = match src.chunk().iter().position(|&b| b == *stop_byte) {
+                Some(i) => i,
+                None => {
+                    return Err(io::Error::new(
+                        io::ErrorKind::InvalidData,
+                        "missing byte array stop byte",
+                    ))
+                }
+            };
 
-            // Remove stop byte.
-            buf.pop();
+            let mut buf = vec![0; len];
+            src.copy_to_slice(&mut buf);
+
+            // Discard the stop byte.
+            src.advance(1);
 
             Ok(buf)
         }
@@ -1059,14 +1126,13 @@ mod tests {
 
         t(&Encoding::External(1), 0x0d)?;
         t(&Encoding::Huffman(vec![0x4e], vec![0]), 0x4e)?;
-        t(&Encoding::Beta(1, 3), 3)?;
 
         Ok(())
     }
 
     #[test]
     fn test_decode_itf8() -> io::Result<()> {
-        fn t(encoding: &Encoding, expected: Itf8) -> io::Result<()> {
+        fn t(encoding: &Encoding, expected: i32) -> io::Result<()> {
             let core_data = [0b10000000];
             let mut core_data_reader = BitReader::new(&core_data[..]);
 
@@ -1123,6 +1189,11 @@ mod tests {
             b"ndls",
         )?;
 
+        assert!(matches!(
+            t(&[0x6e, 0x64, 0x6c, 0x73], &Encoding::ByteArrayStop(0x00, 1), b""),
+            Err(e) if e.kind() == io::ErrorKind::InvalidData
+        ));
+
         Ok(())
     }
 }
