@@ -1,28 +1,31 @@
-use noodles_bam as bam;
-use noodles_sam as sam;
+use noodles_core::Position;
+use noodles_sam::{
+    self as sam,
+    record::{quality_scores::Score, sequence::Base},
+};
 
-use super::{Feature, Flags, NextMateFlags, ReadGroupId, Record, Tag};
+use super::{Feature, Features, Flags, NextMateFlags, Record};
 
 /// A CRAM record builder.
 pub struct Builder {
     id: i64,
     bam_flags: sam::record::Flags,
     flags: Flags,
-    reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
+    reference_sequence_id: Option<usize>,
     read_length: usize,
-    alignment_start: Option<sam::record::Position>,
-    read_group_id: ReadGroupId,
-    read_name: Vec<u8>,
+    alignment_start: Option<Position>,
+    read_group_id: Option<usize>,
+    read_name: Option<sam::record::ReadName>,
     next_mate_flags: NextMateFlags,
-    next_fragment_reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
-    next_mate_alignment_start: Option<sam::record::Position>,
+    next_fragment_reference_sequence_id: Option<usize>,
+    next_mate_alignment_start: Option<Position>,
     template_size: i32,
-    distance_to_next_fragment: i32,
-    tags: Vec<Tag>,
-    bases: Vec<u8>,
-    features: Vec<Feature>,
+    distance_to_next_fragment: Option<usize>,
+    tags: sam::record::Data,
+    bases: sam::record::Sequence,
+    features: Features,
     mapping_quality: Option<sam::record::MappingQuality>,
-    quality_scores: Vec<u8>,
+    quality_scores: sam::record::QualityScores,
 }
 
 impl Builder {
@@ -45,10 +48,7 @@ impl Builder {
     }
 
     /// Sets the reference sequence ID.
-    pub fn set_reference_sequence_id(
-        mut self,
-        reference_sequence_id: bam::record::ReferenceSequenceId,
-    ) -> Self {
+    pub fn set_reference_sequence_id(mut self, reference_sequence_id: usize) -> Self {
         self.reference_sequence_id = Some(reference_sequence_id);
         self
     }
@@ -60,20 +60,20 @@ impl Builder {
     }
 
     /// Sets the alignment start position.
-    pub fn set_alignment_start(mut self, alignment_start: sam::record::Position) -> Self {
+    pub fn set_alignment_start(mut self, alignment_start: Position) -> Self {
         self.alignment_start = Some(alignment_start);
         self
     }
 
     /// Sets the read group ID.
-    pub fn set_read_group_id(mut self, read_group_id: ReadGroupId) -> Self {
-        self.read_group_id = read_group_id;
+    pub fn set_read_group_id(mut self, read_group_id: usize) -> Self {
+        self.read_group_id = Some(read_group_id);
         self
     }
 
     /// Sets the read name.
-    pub fn set_read_name(mut self, read_name: Vec<u8>) -> Self {
-        self.read_name = read_name;
+    pub fn set_read_name(mut self, read_name: sam::record::ReadName) -> Self {
+        self.read_name = Some(read_name);
         self
     }
 
@@ -86,17 +86,14 @@ impl Builder {
     /// Sets the reference sequence ID of the next fragment.
     pub fn set_next_fragment_reference_sequence_id(
         mut self,
-        next_fragment_reference_sequence_id: bam::record::ReferenceSequenceId,
+        next_fragment_reference_sequence_id: usize,
     ) -> Self {
         self.next_fragment_reference_sequence_id = Some(next_fragment_reference_sequence_id);
         self
     }
 
     /// Sets the alignment start position of the next mate.
-    pub fn set_next_mate_alignment_start(
-        mut self,
-        next_mate_alignment_start: sam::record::Position,
-    ) -> Self {
+    pub fn set_next_mate_alignment_start(mut self, next_mate_alignment_start: Position) -> Self {
         self.next_mate_alignment_start = Some(next_mate_alignment_start);
         self
     }
@@ -108,37 +105,37 @@ impl Builder {
     }
 
     /// Sets the distance to the next fragment.
-    pub fn set_distance_to_next_fragment(mut self, distance_to_next_fragment: i32) -> Self {
-        self.distance_to_next_fragment = distance_to_next_fragment;
+    pub fn set_distance_to_next_fragment(mut self, distance_to_next_fragment: usize) -> Self {
+        self.distance_to_next_fragment = Some(distance_to_next_fragment);
         self
     }
 
     /// Sets the tag dictionary.
-    pub fn set_tags(mut self, tags: Vec<Tag>) -> Self {
+    pub fn set_tags(mut self, tags: sam::record::Data) -> Self {
         self.tags = tags;
         self
     }
 
     /// Adds a tag to the tag dictionary.
-    pub fn add_tag(mut self, tag: Tag) -> Self {
-        self.tags.push(tag);
+    pub fn add_tag(mut self, tag: sam::record::data::Field) -> Self {
+        self.tags.insert(tag);
         self
     }
 
     /// Sets the read bases.
-    pub fn set_bases(mut self, bases: Vec<u8>) -> Self {
+    pub fn set_bases(mut self, bases: sam::record::Sequence) -> Self {
         self.bases = bases;
         self
     }
 
     /// Adds a base to the read bases.
-    pub fn add_base(mut self, base: u8) -> Self {
+    pub fn add_base(mut self, base: Base) -> Self {
         self.bases.push(base);
         self
     }
 
     /// Sets the read features.
-    pub fn set_features(mut self, features: Vec<Feature>) -> Self {
+    pub fn set_features(mut self, features: Features) -> Self {
         self.features = features;
         self
     }
@@ -156,13 +153,13 @@ impl Builder {
     }
 
     /// Sets the per-base quality scores.
-    pub fn set_quality_scores(mut self, quality_scores: Vec<u8>) -> Self {
+    pub fn set_quality_scores(mut self, quality_scores: sam::record::QualityScores) -> Self {
         self.quality_scores = quality_scores;
         self
     }
 
     /// Adds a quality score.
-    pub fn add_quality_score(mut self, quality_score: u8) -> Self {
+    pub fn add_quality_score(mut self, quality_score: Score) -> Self {
         self.quality_scores.push(quality_score);
         self
     }
@@ -188,6 +185,7 @@ impl Builder {
             features: self.features,
             mapping_quality: self.mapping_quality,
             quality_scores: self.quality_scores,
+            cigar: Default::default(),
         }
     }
 }
@@ -201,18 +199,18 @@ impl Default for Builder {
             reference_sequence_id: None,
             read_length: 0,
             alignment_start: None,
-            read_group_id: ReadGroupId::default(),
-            read_name: Vec::new(),
+            read_group_id: None,
+            read_name: None,
             next_mate_flags: NextMateFlags::default(),
             next_fragment_reference_sequence_id: None,
             next_mate_alignment_start: None,
             template_size: 0,
-            distance_to_next_fragment: 0,
-            tags: Vec::new(),
-            bases: Vec::new(),
-            features: Vec::new(),
+            distance_to_next_fragment: None,
+            tags: sam::record::Data::default(),
+            bases: sam::record::Sequence::default(),
+            features: Features::default(),
             mapping_quality: None,
-            quality_scores: Vec::new(),
+            quality_scores: sam::record::QualityScores::default(),
         }
     }
 }
@@ -231,13 +229,13 @@ mod tests {
         assert!(builder.reference_sequence_id.is_none());
         assert_eq!(builder.read_length, 0);
         assert!(builder.alignment_start.is_none());
-        assert_eq!(builder.read_group_id, ReadGroupId::default());
-        assert!(builder.read_name.is_empty());
+        assert!(builder.read_group_id.is_none());
+        assert!(builder.read_name.is_none());
         assert_eq!(builder.next_mate_flags, NextMateFlags::default());
         assert!(builder.next_fragment_reference_sequence_id.is_none());
         assert!(builder.next_mate_alignment_start.is_none());
         assert_eq!(builder.template_size, 0);
-        assert_eq!(builder.distance_to_next_fragment, 0);
+        assert!(builder.distance_to_next_fragment.is_none());
         assert!(builder.tags.is_empty());
         assert!(builder.bases.is_empty());
         assert!(builder.features.is_empty());
