@@ -3,7 +3,7 @@ use std::io::{self, Write};
 use byteorder::{LittleEndian, WriteBytesExt};
 use flate2::CrcWriter;
 
-use crate::{container::Block, num::Itf8, writer::num::write_itf8};
+use crate::{container::Block, writer::num::write_itf8};
 
 pub fn write_block<W>(writer: &mut W, block: &Block) -> io::Result<()>
 where
@@ -20,10 +20,11 @@ where
     let block_content_id = block.content_id();
     write_itf8(&mut crc_writer, block_content_id)?;
 
-    let size_in_bytes = block.data().len() as i32;
+    let size_in_bytes = i32::try_from(block.data().len())
+        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
     write_itf8(&mut crc_writer, size_in_bytes)?;
 
-    let uncompressed_data_len = Itf8::try_from(block.uncompressed_len())
+    let uncompressed_data_len = i32::try_from(block.uncompressed_len())
         .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
     write_itf8(&mut crc_writer, uncompressed_data_len)?;
 
