@@ -1,9 +1,7 @@
 use std::io::{self, Write};
 
 use super::write_encoding;
-use crate::{
-    data_container::compression_header::TagEncodingMap, num::Itf8, writer::num::write_itf8,
-};
+use crate::{data_container::compression_header::TagEncodingMap, writer::num::write_itf8};
 
 pub fn write_tag_encoding_map<W>(
     writer: &mut W,
@@ -14,7 +12,8 @@ where
 {
     let mut buf = Vec::new();
 
-    let map_len = tag_encoding_map.len() as Itf8;
+    let map_len = i32::try_from(tag_encoding_map.len())
+        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
     write_itf8(&mut buf, map_len)?;
 
     for (&key, encoding) in tag_encoding_map.iter() {
@@ -22,7 +21,8 @@ where
         write_encoding(&mut buf, encoding)?;
     }
 
-    let data_len = buf.len() as Itf8;
+    let data_len =
+        i32::try_from(buf.len()).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
     write_itf8(writer, data_len)?;
 
     writer.write_all(&buf)?;
