@@ -1,5 +1,3 @@
-mod tag;
-
 use std::{
     collections::HashMap,
     error, fmt,
@@ -9,17 +7,26 @@ use std::{
 use byteorder::WriteBytesExt;
 
 use noodles_bam as bam;
-use noodles_sam as sam;
+use noodles_core::Position;
+use noodles_sam::{
+    self as sam,
+    record::{quality_scores::Score, sequence::Base},
+    AlignmentRecord,
+};
 
 use super::num::write_itf8;
 use crate::{
     container::ReferenceSequenceId,
     data_container::{
-        compression_header::{data_series_encoding_map::DataSeries, Encoding},
+        compression_header::{
+            data_series_encoding_map::DataSeries, preservation_map::tag_ids_dictionary, Encoding,
+        },
         CompressionHeader,
     },
-    num::Itf8,
-    record::{self, feature, Feature, Flags, NextMateFlags},
+    record::{
+        feature::{self, substitution},
+        Feature, Flags, NextMateFlags,
+    },
     BitWriter, Record,
 };
 
@@ -27,7 +34,7 @@ use crate::{
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub enum WriteRecordError {
     MissingDataSeriesEncoding(DataSeries),
-    MissingTagEncoding(record::tag::Key),
+    MissingTagEncoding(tag_ids_dictionary::Key),
     MissingExternalBlock(i32),
 }
 
@@ -50,9 +57,9 @@ impl fmt::Display for WriteRecordError {
 pub struct Writer<'a, W, X> {
     compression_header: &'a CompressionHeader,
     core_data_writer: &'a mut BitWriter<W>,
-    external_data_writers: &'a mut HashMap<Itf8, X>,
+    external_data_writers: &'a mut HashMap<i32, X>,
     reference_sequence_id: ReferenceSequenceId,
-    prev_alignment_start: Option<sam::record::Position>,
+    prev_alignment_start: Option<Position>,
 }
 
 impl<'a, W, X> Writer<'a, W, X>
@@ -63,9 +70,9 @@ where
     pub fn new(
         compression_header: &'a CompressionHeader,
         core_data_writer: &'a mut BitWriter<W>,
-        external_data_writers: &'a mut HashMap<Itf8, X>,
+        external_data_writers: &'a mut HashMap<i32, X>,
         reference_sequence_id: ReferenceSequenceId,
-        initial_alignment_start: Option<sam::record::Position>,
+        initial_alignment_start: Option<Position>,
     ) -> Self {
         Self {
             compression_header,
@@ -78,7 +85,7 @@ where
 
     pub fn write_record(&mut self, record: &Record) -> io::Result<()> {
         self.write_bam_bit_flags(record.bam_flags())?;
-        self.write_cram_bit_flags(record.flags())?;
+        self.write_cram_bit_flags(record.cram_flags())?;
 
         self.write_positional_data(record)?;
 
@@ -136,51 +143,21 @@ where
 
     fn write_positional_data(&mut self, record: &Record) -> io::Result<()> {
         if self.reference_sequence_id.is_many() {
-            let reference_id = record
-                .reference_sequence_id()
-                .map(i32::from)
-                .unwrap_or(bam::record::reference_sequence_id::UNMAPPED);
-            self.write_reference_id(reference_id)?;
+            self.write_reference_id(record.reference_sequence_id())?;
         }
 
         self.write_read_length(record.read_length())?;
-
-        let ap_data_series_delta = self
-            .compression_header
-            .preservation_map()
-            .ap_data_series_delta();
-
-        let alignment_start = if ap_data_series_delta {
-            match (record.alignment_start(), self.prev_alignment_start) {
-                (None, None) => 0,
-                (Some(alignment_start), Some(prev_alignment_start)) => {
-                    i32::from(alignment_start) - i32::from(prev_alignment_start)
-                }
-                _ => {
-                    return Err(io::Error::new(
-                        io::ErrorKind::InvalidData,
-                        format!(
-                            "invalid alignment start ({:?}) or previous alignment start ({:?})",
-                            record.alignment_start(),
-                            self.prev_alignment_start
-                        ),
-                    ));
-                }
-            }
-        } else {
-            record.alignment_start().map(i32::from).unwrap_or_default()
-        };
-
-        self.write_alignment_start(alignment_start)?;
-
-        let read_group = i32::from(record.read_group_id());
-        self.write_read_group(read_group)?;
+        self.write_alignment_start(record.alignment_start)?;
+        self.write_read_group(record.read_group_id())?;
 
         Ok(())
     }
 
-    fn write_reference_id(&mut self, reference_id: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_reference_id(&mut self, reference_sequence_id: Option<usize>) -> io::Result<()> {
+        use bam::record::reference_sequence_id::UNMAPPED;
+
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .reference_id_encoding()
             .ok_or_else(|| {
@@ -188,15 +165,20 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::ReferenceId),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    reference_id,
-                )
-            })
+            })?;
+
+        let reference_id = if let Some(id) = reference_sequence_id {
+            i32::try_from(id).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?
+        } else {
+            UNMAPPED
+        };
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            reference_id,
+        )
     }
 
     fn write_read_length(&mut self, read_length: usize) -> io::Result<()> {
@@ -205,7 +187,7 @@ where
             .data_series_encoding_map()
             .read_lengths_encoding();
 
-        let len = Itf8::try_from(read_length)
+        let len = i32::try_from(read_length)
             .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
 
         encode_itf8(
@@ -216,26 +198,67 @@ where
         )
     }
 
-    fn write_alignment_start(&mut self, alignment_start: Itf8) -> io::Result<()> {
+    fn write_alignment_start(&mut self, alignment_start: Option<Position>) -> io::Result<()> {
+        let ap_data_series_delta = self
+            .compression_header
+            .preservation_map()
+            .ap_data_series_delta();
+
         let encoding = self
             .compression_header
             .data_series_encoding_map()
             .in_seq_positions_encoding();
 
+        let alignment_start_or_delta = if ap_data_series_delta {
+            match (alignment_start, self.prev_alignment_start) {
+                (None, None) => 0,
+                (Some(alignment_start), Some(prev_alignment_start)) => {
+                    let alignment_start = i32::try_from(usize::from(alignment_start))
+                        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+                    let prev_alignment_start = i32::try_from(usize::from(prev_alignment_start))
+                        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+                    alignment_start - prev_alignment_start
+                }
+                _ => {
+                    return Err(io::Error::new(
+                        io::ErrorKind::InvalidInput,
+                        format!(
+                            "invalid alignment start ({:?}) or previous alignment start ({:?})",
+                            alignment_start, self.prev_alignment_start
+                        ),
+                    ));
+                }
+            }
+        } else {
+            i32::try_from(alignment_start.map(usize::from).unwrap_or_default())
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?
+        };
+
         encode_itf8(
             encoding,
             self.core_data_writer,
             self.external_data_writers,
-            alignment_start,
+            alignment_start_or_delta,
         )
     }
 
-    fn write_read_group(&mut self, read_group: Itf8) -> io::Result<()> {
+    fn write_read_group(&mut self, read_group_id: Option<usize>) -> io::Result<()> {
+        // § 10.2 "CRAM positional data" (2021-10-15): "-1 for no group".
+        const MISSING: i32 = -1;
+
         let encoding = self
             .compression_header
             .data_series_encoding_map()
             .read_groups_encoding();
 
+        let read_group = if let Some(id) = read_group_id {
+            i32::try_from(id).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?
+        } else {
+            MISSING
+        };
+
         encode_itf8(
             encoding,
             self.core_data_writer,
@@ -244,8 +267,11 @@ where
         )
     }
 
-    fn write_read_name(&mut self, read_name: &[u8]) -> io::Result<()> {
-        self.compression_header
+    fn write_read_name(&mut self, read_name: Option<&sam::record::ReadName>) -> io::Result<()> {
+        use sam::record::read_name::MISSING;
+
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .read_names_encoding()
             .ok_or_else(|| {
@@ -253,21 +279,20 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::ReadNames),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte_array(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    read_name,
-                )
-            })
+            })?;
+
+        let read_name = read_name.map(|name| name.as_ref()).unwrap_or(MISSING);
+
+        encode_byte_array(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            read_name,
+        )
     }
 
     fn write_mate_data(&mut self, record: &Record) -> io::Result<()> {
-        let flags = record.flags();
-
-        if flags.is_detached() {
+        if record.cram_flags().is_detached() {
             self.write_next_mate_bit_flags(record.next_mate_flags())?;
 
             let preservation_map = self.compression_header.preservation_map();
@@ -276,16 +301,14 @@ where
                 self.write_read_name(record.read_name())?;
             }
 
-            let next_fragment_reference_sequence_id = record
-                .next_fragment_reference_sequence_id()
-                .map(i32::from)
-                .unwrap_or(bam::record::reference_sequence_id::UNMAPPED);
-            self.write_next_fragment_reference_sequence_id(next_fragment_reference_sequence_id)?;
+            self.write_next_fragment_reference_sequence_id(
+                record.next_fragment_reference_sequence_id(),
+            )?;
 
             self.write_next_mate_alignment_start(record.next_mate_alignment_start())?;
             self.write_template_size(record.template_size())?;
-        } else {
-            self.write_distance_to_next_fragment(record.distance_to_next_fragment())?;
+        } else if let Some(distance_to_next_fragment) = record.distance_to_next_fragment() {
+            self.write_distance_to_next_fragment(distance_to_next_fragment)?;
         }
 
         Ok(())
@@ -315,9 +338,12 @@ where
 
     fn write_next_fragment_reference_sequence_id(
         &mut self,
-        next_fragment_reference_sequence_id: Itf8,
+        next_fragment_reference_sequence_id: Option<usize>,
     ) -> io::Result<()> {
-        self.compression_header
+        use bam::record::reference_sequence_id::UNMAPPED;
+
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .next_fragment_reference_sequence_id_encoding()
             .ok_or_else(|| {
@@ -327,20 +353,26 @@ where
                         DataSeries::NextFragmentReferenceSequenceId,
                     ),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    next_fragment_reference_sequence_id,
-                )
-            })
+            })?;
+
+        let raw_next_fragment_reference_sequence_id =
+            if let Some(id) = next_fragment_reference_sequence_id {
+                i32::try_from(id).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?
+            } else {
+                UNMAPPED
+            };
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            raw_next_fragment_reference_sequence_id,
+        )
     }
 
     fn write_next_mate_alignment_start(
         &mut self,
-        next_mate_alignment_start: Option<sam::record::Position>,
+        next_mate_alignment_start: Option<Position>,
     ) -> io::Result<()> {
         let encoding = self
             .compression_header
@@ -353,7 +385,12 @@ where
                 )
             })?;
 
-        let position = next_mate_alignment_start.map(i32::from).unwrap_or_default();
+        let position = i32::try_from(
+            next_mate_alignment_start
+                .map(usize::from)
+                .unwrap_or_default(),
+        )
+        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
 
         encode_itf8(
             encoding,
@@ -363,7 +400,7 @@ where
         )
     }
 
-    fn write_template_size(&mut self, template_size: Itf8) -> io::Result<()> {
+    fn write_template_size(&mut self, template_size: i32) -> io::Result<()> {
         self.compression_header
             .data_series_encoding_map()
             .template_size_encoding()
@@ -385,9 +422,10 @@ where
 
     fn write_distance_to_next_fragment(
         &mut self,
-        distance_to_next_fragment: Itf8,
+        distance_to_next_fragment: usize,
     ) -> io::Result<()> {
-        self.compression_header
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .distance_to_next_fragment_encoding()
             .ok_or_else(|| {
@@ -395,22 +433,26 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::DistanceToNextFragment),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    distance_to_next_fragment,
-                )
-            })
+            })?;
+
+        let n = i32::try_from(distance_to_next_fragment)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            n,
+        )
     }
 
     fn write_tag_data(&mut self, record: &Record) -> io::Result<()> {
+        use bam::writer::record::data::field::put_value;
+
         let preservation_map = self.compression_header.preservation_map();
         let tag_ids_dictionary = preservation_map.tag_ids_dictionary();
 
-        let keys: Vec<_> = record.tags().iter().map(|tag| tag.key()).collect();
+        let keys: Vec<_> = record.tags().values().map(|field| field.into()).collect();
         let tag_line = tag_ids_dictionary
             .iter()
             .enumerate()
@@ -423,21 +465,22 @@ where
                 )
             })?;
 
-        self.write_tag_line(tag_line as Itf8)?;
+        self.write_tag_line(tag_line)?;
 
         let tag_encoding_map = self.compression_header.tag_encoding_map();
+        let mut buf = Vec::new();
 
-        for tag in record.tags() {
-            let id = tag.key().id();
-            let encoding = tag_encoding_map.get(&id).ok_or_else(|| {
+        for field in record.tags().values() {
+            let key: tag_ids_dictionary::Key = field.into();
+            let encoding = tag_encoding_map.get(&key.id()).ok_or_else(|| {
                 io::Error::new(
                     io::ErrorKind::InvalidInput,
-                    WriteRecordError::MissingTagEncoding(tag.key()),
+                    WriteRecordError::MissingTagEncoding(key),
                 )
             })?;
 
-            let mut buf = Vec::new();
-            tag::write_value(&mut buf, tag.value())?;
+            buf.clear();
+            put_value(&mut buf, field.value())?;
 
             encode_byte_array(
                 encoding,
@@ -450,17 +493,20 @@ where
         Ok(())
     }
 
-    fn write_tag_line(&mut self, tag_line: Itf8) -> io::Result<()> {
+    fn write_tag_line(&mut self, tag_line: usize) -> io::Result<()> {
         let encoding = self
             .compression_header
             .data_series_encoding_map()
             .tag_ids_encoding();
 
+        let n =
+            i32::try_from(tag_line).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
         encode_itf8(
             encoding,
             self.core_data_writer,
             self.external_data_writers,
-            tag_line,
+            n,
         )
     }
 
@@ -469,18 +515,16 @@ where
 
         let mut prev_position = 0;
 
-        for feature in record.features() {
-            let position = feature.position() - prev_position;
+        for feature in record.features().iter() {
+            let position = usize::from(feature.position()) - prev_position;
             self.write_feature(feature, position)?;
-            prev_position = feature.position();
+            prev_position = usize::from(feature.position());
         }
 
         self.write_mapping_quality(record.mapping_quality())?;
 
-        let flags = record.flags();
-
-        if flags.are_quality_scores_stored_as_array() {
-            for &score in record.quality_scores() {
+        if record.cram_flags().are_quality_scores_stored_as_array() {
+            for &score in record.quality_scores().as_ref() {
                 self.write_quality_score(score)?;
             }
         }
@@ -489,7 +533,8 @@ where
     }
 
     fn write_number_of_read_features(&mut self, feature_count: usize) -> io::Result<()> {
-        self.compression_header
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .number_of_read_features_encoding()
             .ok_or_else(|| {
@@ -497,20 +542,20 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::NumberOfReadFeatures),
                 )
-            })
-            .and_then(|encoding| {
-                let number_of_read_features = feature_count as Itf8;
+            })?;
 
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    number_of_read_features,
-                )
-            })
+        let number_of_read_features = i32::try_from(feature_count)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            number_of_read_features,
+        )
     }
 
-    fn write_feature(&mut self, feature: &Feature, position: Itf8) -> io::Result<()> {
+    fn write_feature(&mut self, feature: &Feature, position: usize) -> io::Result<()> {
         self.write_feature_code(feature.code())?;
         self.write_feature_position(position)?;
 
@@ -525,8 +570,8 @@ where
                 self.write_base(*base)?;
                 self.write_quality_score(*quality_score)?;
             }
-            Feature::Substitution(_, code) => {
-                self.write_base_substitution_code(*code)?;
+            Feature::Substitution(_, value) => {
+                self.write_base_substitution_code(*value)?;
             }
             Feature::Insertion(_, bases) => {
                 self.write_insertion(bases)?;
@@ -564,13 +609,13 @@ where
             .ok_or_else(|| {
                 io::Error::new(
                     io::ErrorKind::InvalidData,
-                    WriteRecordError::MissingDataSeriesEncoding(DataSeries::InReadPositions),
+                    WriteRecordError::MissingDataSeriesEncoding(DataSeries::ReadFeaturesCodes),
                 )
             })
             .and_then(|encoding| {
-                let feature_code = char::from(code) as Itf8;
+                let feature_code = u8::from(code);
 
-                encode_itf8(
+                encode_byte(
                     encoding,
                     self.core_data_writer,
                     self.external_data_writers,
@@ -579,8 +624,9 @@ where
             })
     }
 
-    fn write_feature_position(&mut self, position: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_feature_position(&mut self, position: usize) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .in_read_positions_encoding()
             .ok_or_else(|| {
@@ -588,19 +634,22 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::InReadPositions),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    position,
-                )
-            })
+            })?;
+
+        let position =
+            i32::try_from(position).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            position,
+        )
     }
 
-    fn write_stretches_of_bases(&mut self, bases: &[u8]) -> io::Result<()> {
-        self.compression_header
+    fn write_stretches_of_bases(&mut self, bases: &[Base]) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .stretches_of_bases_encoding()
             .ok_or_else(|| {
@@ -608,19 +657,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::StretchesOfBases),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte_array(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    bases,
-                )
-            })
+            })?;
+
+        let raw_bases: Vec<_> = bases.iter().copied().map(u8::from).collect();
+
+        encode_byte_array(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            &raw_bases,
+        )
     }
 
-    fn write_stretches_of_quality_scores(&mut self, quality_scores: &[u8]) -> io::Result<()> {
-        self.compression_header
+    fn write_stretches_of_quality_scores(&mut self, quality_scores: &[Score]) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .stretches_of_quality_scores_encoding()
             .ok_or_else(|| {
@@ -630,19 +681,21 @@ where
                         DataSeries::StretchesOfQualityScores,
                     ),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte_array(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    quality_scores,
-                )
-            })
+            })?;
+
+        let scores: Vec<_> = quality_scores.iter().copied().map(u8::from).collect();
+
+        encode_byte_array(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            &scores,
+        )
     }
 
-    fn write_base(&mut self, base: u8) -> io::Result<()> {
-        self.compression_header
+    fn write_base(&mut self, base: Base) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .bases_encoding()
             .ok_or_else(|| {
@@ -650,19 +703,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::Bases),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    base,
-                )
-            })
+            })?;
+
+        let raw_base = u8::from(base);
+
+        encode_byte(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            raw_base,
+        )
     }
 
-    fn write_quality_score(&mut self, quality_score: u8) -> io::Result<()> {
-        self.compression_header
+    fn write_quality_score(&mut self, quality_score: Score) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .quality_scores_encoding()
             .ok_or_else(|| {
@@ -670,19 +725,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::QualityScores),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    quality_score,
-                )
-            })
+            })?;
+
+        let score = u8::from(quality_score);
+
+        encode_byte(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            score,
+        )
     }
 
-    fn write_base_substitution_code(&mut self, code: u8) -> io::Result<()> {
-        self.compression_header
+    fn write_base_substitution_code(&mut self, value: substitution::Value) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .base_substitution_codes_encoding()
             .ok_or_else(|| {
@@ -690,19 +747,36 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::BaseSubstitutionCodes),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    code,
-                )
-            })
+            })?;
+
+        let substitution_matrix = self
+            .compression_header
+            .preservation_map()
+            .substitution_matrix();
+
+        let code = match value {
+            substitution::Value::Bases(reference_base, read_base) => {
+                substitution_matrix.find_code(reference_base, read_base)
+            }
+            substitution::Value::Code(_) => {
+                return Err(io::Error::new(
+                    io::ErrorKind::InvalidInput,
+                    "base substitution cannot be a code on write",
+                ));
+            }
+        };
+
+        encode_byte(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            code,
+        )
     }
 
-    fn write_insertion(&mut self, bases: &[u8]) -> io::Result<()> {
-        self.compression_header
+    fn write_insertion(&mut self, bases: &[Base]) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .insertion_encoding()
             .ok_or_else(|| {
@@ -710,19 +784,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::Insertion),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte_array(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    bases,
-                )
-            })
+            })?;
+
+        let raw_bases: Vec<_> = bases.iter().copied().map(u8::from).collect();
+
+        encode_byte_array(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            &raw_bases,
+        )
     }
 
-    fn write_deletion_length(&mut self, len: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_deletion_length(&mut self, len: usize) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .deletion_lengths_encoding()
             .ok_or_else(|| {
@@ -730,19 +806,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::DeletionLengths),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    len,
-                )
-            })
+            })?;
+
+        let n = i32::try_from(len).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            n,
+        )
     }
 
-    fn write_reference_skip_length(&mut self, len: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_reference_skip_length(&mut self, len: usize) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .reference_skip_length_encoding()
             .ok_or_else(|| {
@@ -750,19 +828,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::ReferenceSkipLength),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    len,
-                )
-            })
+            })?;
+
+        let n = i32::try_from(len).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            n,
+        )
     }
 
-    fn write_soft_clip(&mut self, bases: &[u8]) -> io::Result<()> {
-        self.compression_header
+    fn write_soft_clip(&mut self, bases: &[Base]) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .soft_clip_encoding()
             .ok_or_else(|| {
@@ -770,19 +850,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::SoftClip),
                 )
-            })
-            .and_then(|encoding| {
-                encode_byte_array(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    bases,
-                )
-            })
+            })?;
+
+        let raw_bases: Vec<_> = bases.iter().copied().map(u8::from).collect();
+
+        encode_byte_array(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            &raw_bases,
+        )
     }
 
-    fn write_padding(&mut self, len: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_padding(&mut self, len: usize) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .padding_encoding()
             .ok_or_else(|| {
@@ -790,19 +872,21 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::Padding),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    len,
-                )
-            })
+            })?;
+
+        let n = i32::try_from(len).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            n,
+        )
     }
 
-    fn write_hard_clip(&mut self, len: Itf8) -> io::Result<()> {
-        self.compression_header
+    fn write_hard_clip(&mut self, len: usize) -> io::Result<()> {
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .hard_clip_encoding()
             .ok_or_else(|| {
@@ -810,15 +894,16 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::HardClip),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    self.core_data_writer,
-                    self.external_data_writers,
-                    len,
-                )
-            })
+            })?;
+
+        let n = i32::try_from(len).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+        encode_itf8(
+            encoding,
+            self.core_data_writer,
+            self.external_data_writers,
+            n,
+        )
     }
 
     fn write_mapping_quality(
@@ -851,14 +936,12 @@ where
     }
 
     fn write_unmapped_read(&mut self, record: &Record) -> io::Result<()> {
-        for &base in record.bases() {
+        for &base in record.bases().as_ref() {
             self.write_base(base)?;
         }
 
-        let flags = record.flags();
-
-        if flags.are_quality_scores_stored_as_array() {
-            for &score in record.quality_scores() {
+        if record.cram_flags().are_quality_scores_stored_as_array() {
+            for &score in record.quality_scores().as_ref() {
                 self.write_quality_score(score)?;
             }
         }
@@ -870,7 +953,7 @@ where
 fn encode_byte<W, X>(
     encoding: &Encoding,
     _core_data_writer: &mut BitWriter<W>,
-    external_data_writers: &mut HashMap<Itf8, X>,
+    external_data_writers: &mut HashMap<i32, X>,
     value: u8,
 ) -> io::Result<()>
 where
@@ -897,8 +980,8 @@ where
 fn encode_itf8<W, X>(
     encoding: &Encoding,
     _core_data_writer: &mut BitWriter<W>,
-    external_data_writers: &mut HashMap<Itf8, X>,
-    value: Itf8,
+    external_data_writers: &mut HashMap<i32, X>,
+    value: i32,
 ) -> io::Result<()>
 where
     W: Write,
@@ -924,7 +1007,7 @@ where
 fn encode_byte_array<W, X>(
     encoding: &Encoding,
     core_data_writer: &mut BitWriter<W>,
-    external_data_writers: &mut HashMap<Itf8, X>,
+    external_data_writers: &mut HashMap<i32, X>,
     data: &[u8],
 ) -> io::Result<()>
 where
@@ -945,7 +1028,8 @@ where
             writer.write_all(data)
         }
         Encoding::ByteArrayLen(len_encoding, value_encoding) => {
-            let len = data.len() as Itf8;
+            let len = i32::try_from(data.len())
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
             encode_itf8(len_encoding, core_data_writer, external_data_writers, len)?;
 
             encode_byte_array(
